import logging
from pathlib import Path

log = logging.getLogger(__name__)

WIN_ENCODING = 'ISO-8859-1'

HOME_DIR = Path.home()

PACKAGE_DIR = Path(__file__).absolute().resolve().parent
PACKAGE_NAME = PACKAGE_DIR.name
SRC_DIR = PACKAGE_DIR.parent
if SRC_DIR.name != 'src':
    SRC_DIR = PACKAGE_DIR
REPO_DIR = SRC_DIR.parent
DOCS_DIR = REPO_DIR / 'docs'
IMAGES_DIR = DOCS_DIR / 'images'

DATA_DIR_NAME = f'.{PACKAGE_NAME}-data'
DATA_DIR = PACKAGE_DIR / DATA_DIR_NAME
if not DATA_DIR.is_dir():
    DATA_DIR = REPO_DIR / DATA_DIR_NAME
# canonical data directory to share data between nlpia2 installations
HOME_DATA_DIR = HOME_DIR / DATA_DIR_NAME
if not HOME_DATA_DIR.is_dir():
    # TODO: use tempfiles.tempdir()?
    HOME_DATA_DIR.mkdir(parents=True, exist_ok=True)
if not DATA_DIR.is_dir():
    DATA_DIR = HOME_DATA_DIR


# TODO: share data between users of jupyter hub or tljh
# configurable location to share data between users on the same machine
# canonical data directory to share with other applications
# SHARED_DATA_DIR = Path('/data')




import boto3
from boto3 import session
from botocore.client import Config
from boto3.s3.transfer import S3Transfer

from proai.constants import DO_ACCESS_ID, DO_SECRET_KEY

# Initiate session
session = session.Session()
client = session.client('s3',
                        region_name='sgp1',  # enter your own region_name
                        endpoint_url='https://sgp1.digitaloceanspaces.com',  # enter your own endpoint url

                        aws_access_key_id=DO_ACCESS_ID,
                        aws_secret_access_key=DO_SECRET_KEY)

transfer = S3Transfer(client)

# Uploads a file called 'name-of-file' to your Space called 'name-of-space'
# Creates a new-folder and the file's final name is defined as 'name-of-file'
transfer.upload_file('name-of-file', 'name-of-space', 'new-folder' + "/" + 'new-name-of-file')

# This makes the file you are have specifically uploaded public by default.
response = client.put_object_acl(ACL='public-read', Bucket='name-of-space', Key="%s/%s" % ('new-folder', 'new-name-of-file'))
view raw.py hosted with ❤ by GitHub



""" config file parsing and environment var loading for configuration of your app """
from pathlib import Path


def load_env(filepath='.env'):
    filepath = Path(filepath)
    with filepath.open() as fin:
        lines = fin.readlines()
    var_values = {}
    for line in lines:
        line = line.strip()
        if not line or line.startswith('#'):
            continue
        try:
            name, value = line.split('=')
            name, value = name.strip().upper(), value.strip()
            if len(value) > 1:
                value = value[1:-1] if value[0] in '"\'' and value[-1] == value[0] else value
        except (ValueError, IndexError):
            name, value = None, None
        if name and value:
            var_values[name.upper().strip()] = value
    return var_values



import pandas as pd
import logging
from pathlib import Path
from proai.constants import DATA_DIR

log = logging.getLogger(__name__)


def clean_df_columns(df=None, columns=None, drop_unknown_columns=True):
    r""" Return DataFrame with normalized column names: lower() strip() " "->"_"
    """
    columns = df.columns if columns is None else columns
    if isinstance(columns, str):
        columns = [columns]
    else:
        # df = df if isinstance(df, pd.DataFrame) else None
        columns = list(columns)
    new_column_names = [c.lower().strip().replace(' ', '_') for c in columns]
    column_name_map = dict(zip(columns, new_column_names))
    df = df.rename(columns=column_name_map)
    if drop_unknown_columns:
        unknown_columns = [
            c for c in new_column_names if c.lower().lstrip().startswith('unknown')]
        for c in unknown_columns:
            df.drop(c, inplace=True)
    return df


def make_filepath(filepath,
                  data_dirs=('', DATA_DIR),
                  suffixes='.csv .tsv'.split()):
    """ TODO: strip leading / to try abs and rel path """
    filepath = Path(filepath)
    names = (filepath, filepath.name, filepath.with_suffix(''))
    for pref in data_dirs:  # , SHARED_DATA_DIR)
        for suf in suffixes:
            for name in names:
                for suf2 in [''] + '.gz .zip .bz2'.split():
                    p = Path(pref) / Path(name)
                    # separate branch for if p.suffix p.suffix + suf + suf2
                    if suf:
                        if suf2:
                            p = p.with_suffix(suf + suf2)
                        else:
                            p = p.with_suffix(suf)
                    else:
                        if suf2:
                            p = p.with_suffix(suf2)
                    if p.is_file():
                        return p


def make_df(df_or_filepath,
            column_name_cleaner=clean_df_columns,
            drop_unknown_columns=True,
            low_memory=False,
            index_col=0,
            **kwargs):
    """ Construct DataFrame from filepath or array-like object

    >>> df =  make_df([
    ...     dict(hi='world', bye='planet'),
    ...     dict(hi=1, low=-1),
    ...     ])
    >>> df.shape
    (2, 3)
    >>> df
          hi     bye  low
    0  world  planet  NaN
    1      1     NaN -1.0
    """
    try:
        df_or_filepath = pd.DataFrame(
            data=df_or_filepath.copy(),
            columns=list(df_or_filepath.columns.values),
            index=list(df_or_filepath.index.values))
    except AttributeError:
        pass
    if isinstance(df_or_filepath, (Path, str)):
        p = make_filepath(df_or_filepath)
        df_or_filepath = pd.read_csv(str(p), low_memory=low_memory, index_col=index_col, **kwargs)
    if not isinstance(df_or_filepath, pd.DataFrame):
        try:
            return pd.DataFrame(df_or_filepath,
                                index_col=index_col,
                                low_memory=low_memory, **kwargs)
        except Exception:
            pass
            # look for files [with_suffix('.csv') '.gz', '.csv.gz']
    if column_name_cleaner:
        return column_name_cleaner(df_or_filepath,
                                   drop_unknown_columns=drop_unknown_columns)
    return df_or_filepath


def make_series(series_or_filepath, colnum=-1):
    """ Construct a Series from filepath or array-like object

    >>> series = make_series(DATA_DIR / )
    >>> series.shape
    (200,)
    >>> series.name
    'changed_college_Y2'
    >>> series2 = load_series(series)
    >>> series2.shape == series.shape
    True
    >>> series.name == series2.name
    True
    >>> series.values.sum() == series.values.sum()
    True
    """
    try:
        return pd.Series(
            data=series_or_filepath.copy(),
            index=list(series_or_filepath.index.values))
    except AttributeError:
        if isinstance(series_or_filepath, (Path, str)):
            series_or_filepath = pd.read_csv(series_or_filepath)
    if len(series_or_filepath.shape) == 1:
        return pd.Series(series_or_filepath)
    elif series_or_filepath.shape[1] == 1:
        return pd.Series(series_or_filepath)
    elif series_or_filepath.shape[0] == 1:
        return pd.Series(series_or_filepath.iloc[0])
    else:
        return series_or_filepath[list(series_or_filepath.columns)[colnum]]
    return pd.Series(series_or_filepath)



""" Detectors for the 10 different types of data science variables/data/features

1. numerical (1, 3e8, -2.7182818284590)
2. categorical (red, green, blue)
3. tags: nonexclusive categories such as [fiction, science, novel])
4. ordinal (small, medium, large)
5. date/time/datetime (12/25/2025, 2020-10-26 00:00:00.0001)
6. location (lat/lon, address, zip code, etc)
7. natural language (a description or note with high cardinality)
8. image (*.jpg, *.png, *.bmp)
9. video (*.mp4, *.mpeg, *.avi)
10. audio or time series (chat logs, dolphin chirps, bird song)

"""
import pandas as pd
import numpy as np
from collections.abc import Mapping

NULL_VALUES = set(['None', 'null', "'", '"', ""])
NULL_ZERO_VALUES = NULL_VALUES.union(set(['0'] + ['0.' + z for z in ('0' * i for i in range(10))]))

NULL_VALUES_LOWERED = set((str(x).lower().strip() for x in NULL_VALUES))


def is_null(obj):
    r""" Is the provided object processable as a float

    >>> is_null(np.nan)
    True
    >>> is_null('nan')
    True
    >>> is_null('nuLl')
    True
    >>> is_null(42)
    False
    >>> is_null(None)
    True
    >>> is_null(' None\n   ')
    True
    """
    try:
        obj = float(obj)
        return np.isnan(obj)
    except (ValueError, TypeError):
        pass
    obj = str(obj).strip().lower()
    if obj in NULL_VALUES_LOWERED:
        return True
    return False


def is_number(obj):
    r""" Is the provided object processable as a float

    >>> is_number('NaN')
    True
    >>> is_number('hello world')
    False
    >>> is_number(42)
    True
    """
    try:
        float(obj)
        return True
    except ValueError:
        return False


def is_numerical(
        series: (pd.Series, list, tuple, np.ndarray, dict, Mapping),
        min_purity=1.0):
    r""" Return True if the pd.Series can be processed as a numerical feature for DS

    >>> is_numerical([1, 2, 3])
    True
    >>> is_numerical([1, 2, 3, 'nope'])
    False
    >>> is_numerical([1, 2, 3, 'nope'], min_purity=0.7)
    True
    >>> is_numerical([1, 2, 3, 'None'])
    False
    """
    # TODO: incorporate magic load_series function
    if not isinstance(series, pd.Series):
        series = pd.Series(series)
    mask = series.apply(is_number)
    min_purity = min_purity if isinstance(min_purity, float) else min_purity / len(mask)
    return mask.sum() / len(mask) >= min_purity



import re
from pathlib import Path
from setuptools import find_packages, setup

__version__ = None
# TODO: default requirements here and try/except with loud failure
with Path('requirements.txt').open() as fin:
    install_requires = [r.strip() for r in fin.readlines()]
    # r = install_requires[0]
    # if re.match(r'^#\s*\d{1,2}[.]\d{1,4}.\d{1,4}[rd]?\s*$', r):
    #     __version__ = req.strip().strip('#').strip()
    install_requires = [
	r.strip() for r in install_requires
        if r.strip() and not r.strip().startswith('#')]

__version__ = __version__ or '0.0.8'

setup(
    url='https://gitlab.com/prosocialai/proai',
    author_email='hobson@proai.org',
    name='proai',
    packages=find_packages(where='.'),
    install_requires=install_requires,
    version=__version__,
    description='Tools and AI for building prosocial AI algorithms.',
    author='Hobson Lane (ProAI.org)',
    license='Hippocratic License (MIT + Do No Harm)',
)



#!/usr/bin/env python3
""" AES encryption of strings and bytes """
import sys
import base64

from Crypto.Cipher import AES
from Crypto.Hash import SHA256
from Crypto import Random


def encrypt(key, source, bytes_encoding='utf-8', encode_base64=True):
    if not isinstance(key, str):
        bytes_encoding = None
    if isinstance(key, str):
        key = key.encode(bytes_encoding or 'utf-8')
    if not isinstance(source, str):
        bytes_encoding = None
    if isinstance(source, str):
        source = source.encode(bytes_encoding or 'utf-8')

    # use SHA-256 over our key to get a proper-sized AES key (256-bit?):
    key = SHA256.new(key).digest()
    # generate random IV
    IV = Random.new().read(AES.block_size)
    encryptor = AES.new(key, AES.MODE_CBC, IV)
    # calculate how many bytes of padding are needed
    padding_len = AES.block_size - (len(source) % AES.block_size)
    source += bytes([padding_len]) * padding_len
    encrypted = IV + encryptor.encrypt(source)  # store the IV at the beginning and encrypt
    if encode_base64 or bytes_encoding:
        encrypted = base64.b64encode(encrypted)
    if bytes_encoding:
        encrypted = encrypted.decode(bytes_encoding)
    return encrypted


def decrypt(key, source, bytes_encoding='utf-8', decode_base64=True):
    if not isinstance(key, str):
        bytes_encoding = None
    if isinstance(key, str):
        key = key.encode(bytes_encoding or 'utf-8')
    if not isinstance(source, str):
        bytes_encoding = None

    if decode_base64 or bytes_encoding:
        source = base64.b64decode(source)
    if isinstance(source, str):
        source = source.encode(bytes_encoding or 'utf-8')

    # key and source are both now bytes
    key = SHA256.new(key).digest()  # use SHA-256 over our key to get a proper-sized AES key
    # extract the IV from the beginning
    IV = source[:AES.block_size]
    decryptor = AES.new(key, AES.MODE_CBC, IV)
    decrypted = decryptor.decrypt(source[AES.block_size:])
    # get the padding value from the end
    padding = decrypted[-1]
    if decrypted[-padding:] != bytes([padding]) * padding:
        raise ValueError("Invalid padding...")
    decrypted = decrypted[:-padding]  # remove the padding
    # decrypted is now bytes

    if bytes_encoding:
        return decrypted.decode(bytes_encoding)
    return decrypted


if __name__ == '__main__':
    if len(sys.argv) > 3:
        command = sys.argv[1].strip().strip('-')[0]
        source_filepath = sys.argv[2]
        key = sys.argv[3]
        source = open(source_filepath).read()
        if command == 'e':
            encrypted = encrypt(key=key, source=source)
            print(encrypted, end="")
        elif command == 'd':
            decrypted = decrypt(key=key, source=source)
            print(decrypted, end="")
    else:
        usage = ('USAGE: \n'
                 '       python aes.py -e filename.csv password > filename.csv.aes\n'
                 '       python aes.py -d filename.csv.aes password > filename.csv\n')
        raise ValueError(usage)



""" Text cleaning

- [x] HTML emojies to -> unicode emojies/emoticons
- [x] U+XX html codes -> unicode chars
- [ ] unicode to ascii
- [ ] remove diacritics
"""
import numpy as np
import pandas as pd
from proai.constants import DATA_DIR
from proai.make import make_df


def build_html_translation_dict(html_numbers=None, html_names=None, html_emojies=None):
    if isinstance(html_numbers, pd.Series):
        html_numbers = html_numbers.to_dict()
    if isinstance(html_names, pd.Series):
        html_names = html_names.to_dict()

    html_dict = {f'&#{i};': chr(i) for i in range(256)}
    html_dict.update({f'&#{i:03d};': chr(i) for i in range(256)})

    if html_emojies is None or not len(html_emojies):
        html_emojies = {}
    elif isinstance(html_emojies, pd.Series):
        html_emojies = html_emojies.str.replace('U+', '', regex=False).str.split()
        html_emojies = html_emojies.to_dict()
    elif isinstance(html_emojies, dict):
        html_emojies = dict([
            (k, str(v).replace('U+', '')) for (k, v) in html_emojies.items() if k and v])

    html_codes = []
    for codes in html_emojies:

        html_code = ''
        for c in codes:
            try:
                i = int(c, base=16)
                html_code + f'&#{i};'
            except ValueError:
                pass
        html_codes.append(html_code)
    html_dict.update(zip(html_codes, html_emojies.values()))
    html_dict.update(html_numbers)
    html_dict.update(html_names)
    for k in ('', None, np.nan):
        if k in html_dict:
            del html_dict[k]
    return html_dict


DF_EMOJIES = make_df(DATA_DIR / 'emojies.csv').dropna()
DF_ASCII = make_df(DATA_DIR / 'ascii_extended.csv').dropna()
HTML_NUMBER_SYMBOLS = pd.Series(DF_ASCII['symbol'].values,
                                index=DF_ASCII['html_number'].values)
HTML_NAME_SYMBOLS = pd.Series(DF_ASCII['symbol'].values,
                              index=DF_ASCII['html_name'].values)
HTML_EMOJI_SYMBOLS = pd.Series(DF_EMOJIES['cldr_short_name'].values,
                               index=DF_EMOJIES['browser'])
HTML_DICT = build_html_translation_dict(
    html_numbers=HTML_NUMBER_SYMBOLS,
    html_names=HTML_NAME_SYMBOLS,
    html_emojies=HTML_EMOJI_SYMBOLS
)  # compose_html_dict()
# HTML_BYTES_DICT = dict(
#     [(h.encode(), c.encode()) for (h, c) in HTML_DICT.items()]
# )
